# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/formatting.ipynb (unless otherwise specified).

__all__ = ['Verbosity', 'review_files_table']

# Cell
from typing import List
from pathlib import Path

from devtransfer import utils
from rich.console import Console
from rich import print
from rich.table import Table


class Verbosity:
    DETAIL = 2
    INFO = 1
    NONSET = 0


def review_files_table(
        title='',
        root_dir: Path = Path('.'),
        files: List = [],
        sort = True,
        include_absolute=False
        ):

    final_paths = [utils.resolve_path(x) for x in files]
    if sort:
        final_paths = sorted(final_paths, key=lambda x: x.name)
    relative_paths = [utils.resolve_path(x).relative_to(root_dir.resolve()) for x in final_paths]
    relative_paths = [f"{x}/" if x.is_dir() else f"{x}" for x in relative_paths]
    valid_sizes = [":white_check_mark:" if utils.verify_file_size(x) else ":x:" for x in final_paths]
    final_paths = [utils.resolve_path(x) for x in final_paths]
    file_sizes = [utils.file_size_display(x, humanize=False) for x in final_paths]
    abs_paths = [str(x) for x in final_paths]
    only_absolute = False
    if len(abs_paths) == 1:
        only_absolute = True
    table = Table(title=title)
    table.add_column("#")
    table.add_column("Path", justify="left", style="green", no_wrap=True)
    if include_absolute:
        table.add_column("Absolute Path", justify="left", style="magenta", no_wrap=True)
    table.add_column("Uploadable")
    table.add_column("Size")

    cumulative_file_size = 0
    for i, row in enumerate(zip(relative_paths, abs_paths)):
        i_str = f"{i}"
        relative_path = row[0]
        abs_path = row[1]
        if only_absolute:
            relative_path = abs_path
        valid_size = valid_sizes[i]
        cumulative_file_size += file_sizes[i]
        file_size_display = utils.human_size(file_sizes[i])
        if include_absolute:
            table.add_row(i_str, relative_path, abs_path, valid_size, file_size_display)
        else:
            table.add_row(i_str, relative_path, valid_size, file_size_display)
    console = Console()
    console.print(table)
    cumulative_file_size_display = utils.human_size(cumulative_file_size)
    print(f"Total size: {cumulative_file_size_display} ({cumulative_file_size})\n")